@@ -9,21 +9,23 @@ package Spreadsheet::Read;
 =head1 SYNOPSIS
 
  use Spreadsheet::Read;
- my $ref = ReadData ("test.csv", sep => ";");
- my $ref = ReadData ("test.sxc");
- my $ref = ReadData ("test.ods");
- my $ref = ReadData ("test.xls");
- my $ref = ReadData ("test.xlsx");
- my $ref = ReadData ($fh, parser => "xls");
+ my $book  = ReadData ("test.csv", sep => ";");
+ my $book  = ReadData ("test.sxc");
+ my $book  = ReadData ("test.ods");
+ my $book  = ReadData ("test.xls");
+ my $book  = ReadData ("test.xlsx");
+ my $book  = ReadData ($fh, parser => "xls");
 
- my $a3 = $ref->[1]{A3}, "\n"; # content of field A3 of sheet 1
+ my $sheet = $book->[1];             # first datasheet
+ my $cell  = $book->[1]{A3};         # content of field A3 of sheet 1
+ my $cell  = $book->[1]{cell}[1][3]; # same, unformatted
 
 =cut
 
 use strict;
 use warnings;
 
-our $VERSION = "0.41";
+our $VERSION = "0.54";
 sub  Version { $VERSION }
 
 use Carp;
@@ -36,27 +38,41 @@ use File::Temp   qw( );
 use Data::Dumper;
 
 my @parsers = (
-    [ csv	=> "Text::CSV_XS"		],
-    [ csv	=> "Text::CSV_PP"		], # Version 1.05 and up
-    [ csv	=> "Text::CSV"			], # Version 1.00 and up
-    [ ods	=> "Spreadsheet::ReadSXC"	],
-    [ sxc	=> "Spreadsheet::ReadSXC"	],
-    [ xls	=> "Spreadsheet::ParseExcel"	],
-    [ xlsx	=> "Spreadsheet::XLSX"		],
-    [ prl	=> "Spreadsheet::Perl"		],
+    [ csv  => "Text::CSV_XS",              "0.71"  ],
+    [ csv  => "Text::CSV_PP",              "1.17"  ],
+    [ csv  => "Text::CSV",                 "1.17"  ],
+    [ ods  => "Spreadsheet::ReadSXC",      "0.20"  ],
+    [ sxc  => "Spreadsheet::ReadSXC",      "0.20"  ],
+    [ xls  => "Spreadsheet::ParseExcel",   "0.34"  ],
+    [ xlsx => "Spreadsheet::ParseXLSX",    "0.13"  ],
+    [ xlsx => "Spreadsheet::XLSX",         "0.13"  ],
+    [ prl  => "Spreadsheet::Perl",         ""      ],
 
     # Helper modules
-    [ ios	=> "IO::Scalar"			],
+    [ ios  => "IO::Scalar",                ""      ],
+    [ dmp  => "Data::Peek",                ""      ],
     );
 my %can = map { $_->[0] => 0 } @parsers;
 for (@parsers) {
-    my ($flag, $mod) = @$_;
+    my ($flag, $mod, $vsn) = @$_;
     $can{$flag} and next;
-    eval "require $mod; \$can{\$flag} = '$mod'";
+    eval "require $mod; $vsn and ${mod}->VERSION ($vsn); \$can{\$flag} = '$mod'";
     }
 $can{sc} = __PACKAGE__;	# SquirelCalc is built-in
 
 my $debug = 0;
+my %def_opts = (
+    rc      => 1,
+    cells   => 1,
+    attr    => 0,
+    clip    => undef, # $opt{cells};
+    strip   => 0,
+    dtfmt   => "yyyy-mm-dd", # Format 14
+    debug   => 0,
+    parser  => undef,
+    sep     => undef,
+    quote   => undef,
+    );
 my @def_attr = (
     type    => "text",
     fgcolor => undef,
@@ -78,6 +94,17 @@ my @def_attr = (
 
 # Helper functions
 
+sub _dump
+{
+    my ($label, $ref) = @_;
+    if ($can{dmp}) {
+	print STDERR Data::Peek::DDumper ({ $label => $ref });
+	}
+    else {
+	print STDERR Data::Dumper->Dump ([$ref], [$label]);
+	}
+    } # _dump
+
 sub _parser
 {
     my $type = shift		or  return "";
@@ -88,6 +115,7 @@ sub _parser
     $type eq "oo"		and return "sxc";
     $type eq "ods"		and return "sxc";
     $type eq "openoffice"	and return "sxc";
+    $type eq "libreoffice"	and return "sxc";
     $type eq "perl"		and return "prl";
     $type eq "squirelcalc"	and return "sc";
     return exists $can{$type} ? $type : "";
@@ -127,7 +155,7 @@ sub cell2cr
     ($c, $r);
     } # cell2cr
 
-# my @row = cellrow ($ss->[1], 1);
+# my @row = cellrow ($book->[1], 1);
 sub cellrow
 {
     my $sheet = shift or return;
@@ -139,7 +167,7 @@ sub cellrow
     map { $s->[$_][$row] } 1..$sheet->{maxcol};
     } # cellrow
 
-# my @row = row ($ss->[1], 1);
+# my @row = row ($book->[1], 1);
 sub row
 {
     my $sheet = shift or return;
@@ -151,7 +179,7 @@ sub row
     } # row
 
 # Convert {cell}'s [column][row] to a [row][column] list
-# my @rows = rows ($ss->[1]);
+# my @rows = rows ($book->[1]);
 sub rows
 {
     my $sheet = shift or return;
@@ -165,22 +193,24 @@ sub rows
 	} 1..$sheet->{maxrow};
     } # rows
 
-# If option "clip" is set, remove the trailing lines and
+# If option "clip" is set, remove the trailing rows and
 # columns in each sheet that contain no visible data
 sub _clipsheets
 {
     my ($opt, $ref) = @_;
 
-    if (my $s = $opt->{strip}) {
+    if (my $s = $opt->{strip} and $ref->[0]{sheets}) {
 	foreach my $sheet (1 .. $ref->[0]{sheets}) {
 	    my $ss = $ref->[$sheet];
+	    $ss->{maxrow} && $ss->{maxcol} or next;
 	    foreach my $row (1 .. $ss->{maxrow}) {
 		foreach my $col (1 .. $ss->{maxcol}) {
-		    defined $ss->{cell}[$col][$row] or next;
-		    $s & 2 && $ss->{cell}[$col][$row] =~ s/\s+$// and
-			$ss->{cr2cell ($col, $row)}   =~ s/\s+$//;
-		    $s & 1 && $ss->{cell}[$col][$row] =~ s/^\s+// and
-			$ss->{cr2cell ($col, $row)}   =~ s/^\s+//;
+		    for (($opt->{cells} ? $ss->{cell}[$col][$row] : ()),
+		         ($opt->{rc} ? $ss->{cr2cell ($col, $row)} : ())) {
+			defined or next;
+		        $s & 2 and s/\s+$//;
+			$s & 1 and s/^\s+//;
+			}
 		    }
 		}
 	    }
@@ -202,7 +232,7 @@ sub _clipsheets
 	    }
 	$ss->{maxcol} or $ss->{maxrow} = 0;
 
-	# Remove trailing empty lines
+	# Remove trailing empty rows
 	while ($ss->{maxrow} and not (
 		grep { defined && m/\S/ }
 		map  { $ss->{cell}[$_][$ss->{maxrow}] }
@@ -217,16 +247,29 @@ sub _clipsheets
     $ref;
     } # _clipsheets
 
-sub _xls_color {
-    my ($clr, @clr) = @_;
-    defined $clr               or  return undef;
-    @clr == 0 && $clr == 32767 and return undef; # Default fg color
-    @clr == 2 && $clr ==     0 and return undef; # No fill bg color
-    @clr == 2 && $clr ==     1 and ($clr, @clr) = ($clr[0]);
-    @clr and return undef; # Don't know what to do with this
-    "#" . lc Spreadsheet::ParseExcel->ColorIdxToRGB ($clr);
+# Convert a single color (index) to a color
+sub _xls_color
+{
+    my $clr = shift;
+    defined $clr		or  return undef;
+    $clr eq "#000000"		and return undef;
+    $clr =~ m/^#[0-9a-fA-F]+$/	and return lc $clr;
+    $clr == 0 || $clr == 32767	and return undef; # Default fg color
+    return "#" . lc Spreadsheet::ParseExcel->ColorIdxToRGB ($clr);
     } # _xls_color
 
+# Convert a fill [ $pattern, $front_color, $back_color ] to a single background
+sub _xls_fill
+{
+    my ($p, $fg, $bg) = @_;
+    defined $p			or  return undef;
+    $p == 32767			and return undef; # Default fg color
+    $p == 0 && !defined $bg	and return undef; # No fill bg color
+    $p == 1			and return _xls_color ($fg);
+    $bg < 8 || $bg > 63		and return undef; # see Workbook.pm#106
+    return _xls_color ($bg);
+    } # _xls_fill
+
 sub ReadData
 {
     my $txt = shift	or  return;
@@ -236,16 +279,21 @@ sub ReadData
 	   if (ref $_[0] eq "HASH")  { %opt = %{shift @_} }
 	elsif (@_ % 2 == 0)          { %opt = @_          }
 	}
-    defined $opt{rc}	or $opt{rc}	= 1;
-    defined $opt{cells}	or $opt{cells}	= 1;
-    defined $opt{attr}	or $opt{attr}	= 0;
-    defined $opt{clip}	or $opt{clip}	= $opt{cells};
-    defined $opt{strip}	or $opt{strip}	= 0;
-    defined $opt{dtfmt} or $opt{dtfmt}	= "yyyy-mm-dd"; # Format 14
+
+    exists $opt{rc}	or $opt{rc}	= $def_opts{rc};
+    exists $opt{cells}	or $opt{cells}	= $def_opts{cells};
+    exists $opt{attr}	or $opt{attr}	= $def_opts{attr};
+    exists $opt{clip}	or $opt{clip}	= $opt{cells};
+    exists $opt{strip}	or $opt{strip}	= $def_opts{strip};
+    exists $opt{dtfmt}	or $opt{dtfmt}	= $def_opts{dtfmt};
 
     # $debug = $opt{debug} // 0;
-    $debug = defined $opt{debug} ? $opt{debug} : 0;
-    $debug > 4 and print STDERR Data::Dumper->Dump ([\%opt],["Options"]);
+    $debug = defined $opt{debug} ? $opt{debug} : $def_opts{debug};
+    $debug > 4 and _dump (Options => \%opt);
+
+    my %parser_opts = map { $_ => $opt{$_} }
+		      grep { !exists $def_opts{$_} }
+		      keys %opt;
 
     my $io_ref = ref ($txt) =~ m/GLOB|IO/ ? $txt : undef;
     my $io_fil = $io_ref ? 0 : do { no warnings "newline"; -f $txt ? 1 : 0 };
@@ -262,11 +310,11 @@ sub ReadData
 
 	$debug and print STDERR "Opening CSV $label\n";
 
-	my $csv;
 	my @data = (
 	    {	type	=> "csv",
 		parser  => $can{csv},
 		version	=> $can{csv}->VERSION,
+		error	=> undef,
 		quote   => '"',
 		sepchar => ',',
 		sheets	=> 1,
@@ -309,11 +357,14 @@ sub ReadData
 	    $in = $txt;	# Now pray ...
 	    }
 	$debug > 1 and print STDERR "CSV sep_char '$sep', quote_char '$quo'\n";
-	$csv = $can{csv}->new ({
+	my $csv = $can{csv}->new ({
+	    %parser_opts,
+
 	    sep_char       => ($data[0]{sepchar} = $sep),
 	    quote_char     => ($data[0]{quote}   = $quo),
 	    keep_meta_info => 1,
 	    binary         => 1,
+	    auto_diag      => 1,
 	    }) or croak "Cannot create a csv ('$sep', '$quo') parser!";
 
 	while (my $row = $csv->getline ($in)) {
@@ -329,7 +380,7 @@ sub ReadData
 		$opt{attr}  and $data[1]{attr}[$c + 1][$r] = { @def_attr };
 		}
 	    }
-	$csv->eof () or $csv->error_diag;
+	$csv->eof () or $data[0]{error} = [ $csv->error_diag ];
 	close $in;
 
 	for (@{$data[1]{cell}}) {
@@ -367,22 +418,28 @@ sub ReadData
 	$debug and print STDERR "Opening $parse_type \$txt\n";
 	if ($io_ref) {
 	    $oBook = $parse_type eq "XLSX"
-		? Spreadsheet::XLSX::Workbook->Parse ($io_ref)
-		: Spreadsheet::ParseExcel::Workbook->Parse ($io_ref);
+		? $can{xlsx} =~ m/::XLSX/
+		? Spreadsheet::XLSX->new ($io_ref)
+		: Spreadsheet::ParseXLSX->new (%parser_opts)->parse ($io_ref)
+		: Spreadsheet::ParseExcel->new (%parser_opts)->Parse ($io_ref);
 	    }
 	else {
 	    $oBook = $parse_type eq "XLSX"
+		? $can{xlsx} =~ m/::XLSX/
 		? Spreadsheet::XLSX->new ($txt)
-		: Spreadsheet::ParseExcel::Workbook->Parse ($txt);
+		: Spreadsheet::ParseXLSX->new (%parser_opts)->parse ($txt)
+		: Spreadsheet::ParseExcel->new (%parser_opts)->Parse ($txt);
 	    }
-	$oBook or return;
-	$debug > 8 and print STDERR Data::Dumper->Dump ([$oBook],["oBook"]);
+	$debug > 8 and _dump (oBook => $oBook);
 	my @data = ( {
 	    type	=> lc $parse_type,
 	    parser	=> $can{lc $parse_type},
 	    version	=> $parse_type eq "XLSX"
+			 ? $can{xlsx} =~ m/::XLSX/
 			 ? $Spreadsheet::XLSX::VERSION
+			 : $Spreadsheet::ParseXLSX::VERSION
 			 : $Spreadsheet::ParseExcel::VERSION,
+	    error	=> undef,
 	    sheets	=> $oBook->{SheetCount} || 0,
 	    sheet	=> {},
 	    } );
@@ -395,7 +452,9 @@ sub ReadData
 	    );
 	$oBook->{FormatStr}{$_} = $def_fmt{$_} for keys %def_fmt;
 	my $oFmt = $parse_type eq "XLSX"
+	    ? $can{xlsx} =~ m/::XLSX/
 	    ? Spreadsheet::XLSX::Fmt2007->new
+	    : Spreadsheet::ParseExcel::FmtDefault->new
 	    : Spreadsheet::ParseExcel::FmtDefault->new;
 
 	$debug and print STDERR "\t$data[0]{sheets} sheets\n";
@@ -476,8 +535,8 @@ sub ReadData
 				type    => lc $oWkC->{Type},
 				enc     => $oWkC->{Code},
 				merged  => $oWkC->{Merged} || 0,
-				hidden  => $FmT->{Hidden},
-				locked  => $FmT->{Lock},
+				hidden  => $FmT->{Hidden}  || 0,
+				locked  => $FmT->{Lock}    || 0,
 				format  => $fmi,
 				halign  => [ undef, qw( left center right
 					   fill justify ), undef,
@@ -491,7 +550,7 @@ sub ReadData
 				italic  => $FnT->{Italic},
 				uline   => $FnT->{Underline},
 				fgcolor => _xls_color ($FnT->{Color}),
-				bgcolor => _xls_color (@{$FmT->{Fill}}),
+				bgcolor => _xls_fill  (@{$FmT->{Fill}}),
 				};
 			    }
 			}
@@ -532,6 +591,7 @@ sub ReadData
 	    {	type	=> "sc",
 		parser	=> "Spreadsheet::Read",
 		version	=> $VERSION,
+		error	=> undef,
 		sheets	=> 1,
 		sheet	=> { sheet => 1 },
 		},
@@ -550,7 +610,7 @@ sub ReadData
 		}
 	    s/^r([0-9]+)c([0-9]+)\s*=\s*// or next;
 	    my ($c, $r) = map { $_ + 1 } $2, $1;
-	    if (m/.* {(.*)}$/ or m/"(.*)"/) {
+	    if (m/.* \{(.*)}$/ or m/"(.*)"/) {
 		my $cell = cr2cell ($c, $r);
 		$opt{rc}    and $data[1]{cell}[$c][$r] = $1;
 		$opt{cells} and $data[1]{$cell} = $1;
@@ -569,7 +629,7 @@ sub ReadData
     if ($opt{parser} ? _parser ($opt{parser}) eq "sxc"
 		     : ($txt =~ m/^<\?xml/ or -f $txt)) {
 	$can{sxc} or croak "Spreadsheet::ReadSXC not installed";
-	my $sxc_options = { OrderBySheet => 1 }; # New interface 0.20 and up
+	my $sxc_options = { %parser_opts, OrderBySheet => 1 }; # New interface 0.20 and up
 	my $sxc;
 	   if ($txt =~ m/\.(sxc|ods)$/i) {
 	    $debug and print STDERR "Opening \U$1\E $txt\n";
@@ -590,12 +650,13 @@ sub ReadData
 	    }
 	!$sxc && $txt =~ m/^<\?xml/i and
 	    $sxc = Spreadsheet::ReadSXC::read_xml_string ($txt, $sxc_options);
-	$debug > 8 and print STDERR Data::Dumper->Dump ([$sxc],["sxc"]);
+	$debug > 8 and _dump (sxc => $sxc);
 	if ($sxc) {
 	    my @data = ( {
 		type	=> "sxc",
 		parser	=> "Spreadsheet::ReadSXC",
 		version	=> $Spreadsheet::ReadSXC::VERSION,
+		error	=> undef,
 		sheets	=> 0,
 		sheet	=> {},
 		} );
@@ -652,13 +713,12 @@ Spreadsheet::Read tries to transparently read *any* spreadsheet and
 return its content in a universal manner independent of the parsing
 module that does the actual spreadsheet scanning.
 
-For OpenOffice this module uses Spreadsheet::ReadSXC
+For OpenOffice and/or LibreOffice this module uses Spreadsheet::ReadSXC
 
-For Microsoft Excel this module uses Spreadsheet::ParseExcel or
-Spreadsheet::XLSX
+For Microsoft Excel this module uses Spreadsheet::ParseExcel,
+Spreadsheet::ParseXLSX, or Spreadsheet::XLSX.
 
-For CSV this module uses Text::CSV_XS (0.29 or up required, 0.73 or
-up preferred) or Text::CSV_PP (1.05 or up required).
+For CSV this module uses Text::CSV_XS or Text::CSV_PP.
 
 For SquirrelCalc there is a very simplistic built-in parser
 
@@ -666,32 +726,33 @@ For SquirrelCalc there is a very simplistic built-in parser
 
 The data is returned as an array reference:
 
-  $ref = [
- 	# Entry 0 is the overall control hash
- 	{ sheets  => 2,
-	  sheet   => {
-	    "Sheet 1"	=> 1,
-	    "Sheet 2"	=> 2,
-	    },
-	  type    => "xls",
-	  parser  => "Spreadsheet::ParseExcel",
-	  version => 0.26,
-	  },
- 	# Entry 1 is the first sheet
- 	{ label  => "Sheet 1",
- 	  maxrow => 2,
- 	  maxcol => 4,
- 	  cell   => [ undef,
-	    [ undef, 1 ],
-	    [ undef, undef, undef, undef, undef, "Nugget" ],
-	    ],
- 	  A1     => 1,
- 	  B5     => "Nugget",
- 	  },
- 	# Entry 2 is the second sheet
- 	{ label => "Sheet 2",
- 	  :
- 	:
+  $book = [
+      # Entry 0 is the overall control hash
+      { sheets  => 2,
+        sheet   => {
+          "Sheet 1"  => 1,
+          "Sheet 2"  => 2,
+          },
+        type    => "xls",
+        parser  => "Spreadsheet::ParseExcel",
+        version => 0.59,
+	error	=> undef,
+        },
+      # Entry 1 is the first sheet
+      { label   => "Sheet 1",
+        maxrow  => 2,
+        maxcol  => 4,
+        cell    => [ undef,
+          [ undef, 1 ],
+          [ undef, undef, undef, undef, undef, "Nugget" ],
+          ],
+        A1      => 1,
+        B5      => "Nugget",
+        },
+      # Entry 2 is the second sheet
+      { label   => "Sheet 2",
+        :
+        :
 
 To keep as close contact to spreadsheet users, row and column 1 have
 index 1 too in the C<cell> element of the sheet hash, so cell "A1" is
@@ -705,33 +766,33 @@ The control hash (the first entry in the returned array ref), contains
 some spreadsheet meta-data. The entry C<sheet> is there to be able to find
 the sheets when accessing them by name:
 
-  my %sheet2 = %{$ref->[$ref->[0]{sheet}{"Sheet 2"}]};
+  my %sheet2 = %{$book->[$book->[0]{sheet}{"Sheet 2"}]};
 
 =head2 Functions
 
 =over 2
 
-=item my $ref = ReadData ($source [, option => value [, ... ]]);
+=item my $book = ReadData ($source [, option => value [, ... ]]);
 
-=item my $ref = ReadData ("file.csv", sep => ',', quote => '"');
+=item my $book = ReadData ("file.csv", sep => ',', quote => '"');
 
-=item my $ref = ReadData ("file.xls", dtfmt => "yyyy-mm-dd");
+=item my $book = ReadData ("file.xls", dtfmt => "yyyy-mm-dd");
 
-=item my $ref = ReadData ("file.ods");
+=item my $book = ReadData ("file.ods");
 
-=item my $ref = ReadData ("file.sxc");
+=item my $book = ReadData ("file.sxc");
 
-=item my $ref = ReadData ("content.xml");
+=item my $book = ReadData ("content.xml");
 
-=item my $ref = ReadData ($content);
+=item my $book = ReadData ($content);
 
-=item my $ref = ReadData ($fh, parser => "xls");
+=item my $book = ReadData ($fh, parser => "xls");
 
 Tries to convert the given file, string, or stream to the data
 structure described above.
 
 Processing Excel data from a stream or content is supported through
-a File::Temp temporary file or IO::Scalar when available.  
+a File::Temp temporary file or IO::Scalar when available.
 
 ReadSXC does preserve sheet order as of version 0.20.
 
@@ -743,7 +804,8 @@ Currently supported options are:
 
 Force the data to be parsed by a specific format. Possible values are
 C<csv>, C<prl> (or C<perl>), C<sc> (or C<squirelcalc>), C<sxc> (or C<oo>,
-C<ods>, C<openoffice>) C<xls> (or C<excel>), and C<xlsx> (or C<excel2007>).
+C<ods>, C<openoffice>, C<libreoffice>) C<xls> (or C<excel>), and C<xlsx>
+(or C<excel2007>).
 
 When parsing streams, instead of files, it is highly recommended to pass
 this option.
@@ -763,8 +825,10 @@ See L<Cell Attributes> below.
 
 =item clip
 
-If set, C<ReadData ()> will remove all trailing lines and columns per
-sheet that have no visual data.
+If set, C<ReadData ()> will remove all trailing rows and columns per
+sheet that have no visual data. If a sheet has no data at all, the
+sheet will be skipped entirely when this attribute is true.
+
 This option is only valid if C<cells> is true. The default value is
 true if C<cells> is true, and false otherwise.
 
@@ -805,12 +869,15 @@ happened to be at the time the user saved the file.
 
 Enable some diagnostic messages to STDERR.
 
-The value determines how much diagnostics are dumped (using Data::Dumper).
+The value determines how much diagnostics are dumped (using Data::Peek).
 A value of 9 and higher will dump the entire structure from the back-end
 parser.
 
 =back
 
+All other attributes/options will be passed to the underlying parser if
+that parser supports attributes.
+
 =back
 
 =head2 Using CSV
@@ -823,6 +890,13 @@ Text::CSV_PP) is able to automatically detect and use C<\r> line endings).
 CSV can parse streams too, but be sure to pass C<sep> and/or C<quote> if
 these do not match the default C<,> and C<">.
 
+When an error is found in the CSV, it is automatically reported (to STDERR).
+The structure will store the error in C<< $ss->[0]{error} >> as anonymous
+list returned by C<< $csv->error_diag >>. See Text::CSV_XS for documentation.
+
+ my $ss = ReadData ("bad.csv");
+ $ss->[0]{error} and say $ss->[0]{error}[1];
+
 =head2 Functions
 
 =over 4
@@ -843,31 +917,31 @@ pair (1 based):
   my ($col, $row) = cell2cr ("D14"); # returns ( 4, 14)
   my ($col, $row) = cell2cr ("AB4"); # returns (28,  4)
 
-=item my @row = row ($ref, $row)
+=item my @row = row ($sheet, $row)
 
-=item my @row = Spreadsheet::Read::row ($ss->[1], 3)
+=item my @row = Spreadsheet::Read::row ($book->[1], 3)
 
-Get full row of formatted values (like C<< $ss->{A3} .. $ss->{G3} >>)
+Get full row of formatted values (like C<< $sheet->{A3} .. $sheet->{G3} >>)
 
 Note that the indexes in the returned list are 0-based.
 
 C<row ()> is not imported by default, so either specify it in the
 use argument list, or call it fully qualified.
 
-=item my @row = cellrow ($ref, $row)
+=item my @row = cellrow ($book, $row)
 
-=item my @row = Spreadsheet::Read::cellrow ($ss->[1], 3)
+=item my @row = Spreadsheet::Read::cellrow ($book->[1], 3)
 
-Get full row of unformatted values (like C<< $ss->{cell}[1][3] .. $ss->{cell}[7][3] >>)
+Get full row of unformatted values (like C<< $sheet->{cell}[1][3] .. $sheet->{cell}[7][3] >>)
 
 Note that the indexes in the returned list are 0-based.
 
 C<cellrow ()> is not imported by default, so either specify it in the
 use argument list, or call it fully qualified.
 
-=item my @rows = rows ($ref)
+=item my @rows = rows ($book)
 
-=item my @rows = Spreadsheet::Read::rows ($ss->[1])
+=item my @rows = Spreadsheet::Read::rows ($book->[1])
 
 Convert C<{cell}>'s C<[column][row]> to a C<[row][column]> list.
 
@@ -954,6 +1028,109 @@ is made to analyze and store field attributes like this:
 This has now been partially implemented, mainly for Excel, as the other
 parsers do not (yet) support all of that. YMMV.
 
+=head1 TOOLS
+
+This modules comes with a few tools that perform tasks from the FAQ, like
+"How do I select only column D through F from sheet 2 into a CSV file?"
+
+If the module was installed without the tools, you can find them here:
+  http://repo.or.cz/w/Spreadsheet-Read.git/tree/HEAD:/examples
+
+=head2 C<xlscat>
+
+Show (parts of) a spreadsheet in plain text, CSV, or HTML
+
+ usage: xlscat   [-s <sep>] [-L] [-n] [-A] [-u] [Selection] file.xls
+                 [-c | -m]                 [-u] [Selection] file.xls
+                  -i                            [-S sheets] file.xls
+     Generic options:
+        -v[#]       Set verbose level (xlscat/xlsgrep)
+        -d[#]       Set debug   level (Spreadsheet::Read)
+        -u          Use unformatted values
+        --noclip    Do not strip empty sheets and
+                    trailing empty rows and columns
+        -e <enc>    Set encoding for input and output
+        -b <enc>    Set encoding for input
+        -a <enc>    Set encoding for output
+     Input CSV:
+        --in-sep=c  Set input sep_char for CSV
+     Input XLS:
+        --dtfmt=fmt Specify the default date format to replace 'm-d-yy'
+                    the default replacement is 'yyyy-mm-dd'
+     Output Text (default):
+        -s <sep>    Use separator <sep>. Default '|', \n allowed
+        -L          Line up the columns
+        -n          Number lines (prefix with column number)
+        -A          Show field attributes in ANSI escapes
+     Output Index only:
+        -i          Show sheet names and size only
+     Output CSV:
+        -c          Output CSV, separator = ','
+        -m          Output CSV, separator = ';'
+     Output HTML:
+        -H          Output HTML
+     Selection:
+        -S <sheets> Only print sheets <sheets>. 'all' is a valid set
+                    Default only prints the first sheet
+        -R <rows>   Only print rows    <rows>. Default is 'all'
+        -C <cols>   Only print columns <cols>. Default is 'all'
+        -F <flds>   Only fields <flds> e.g. -FA3,B16
+
+=head2 C<xlsgrep>
+
+Show (parts of) a spreadsheet that match a pattern in plain text, CSV, or HTML
+
+ usage: xlsgrep  [-s <sep>] [-L] [-n] [-A] [-u] [Selection] pattern file.xls
+                 [-c | -m]                 [-u] [Selection] pattern file.xls
+                  -i                            [-S sheets] pattern file.xls
+     Generic options:
+        -v[#]       Set verbose level (xlscat/xlsgrep)
+        -d[#]       Set debug   level (Spreadsheet::Read)
+        -u          Use unformatted values
+        --noclip    Do not strip empty sheets and
+                    trailing empty rows and columns
+        -e <enc>    Set encoding for input and output
+        -b <enc>    Set encoding for input
+        -a <enc>    Set encoding for output
+     Input CSV:
+        --in-sep=c  Set input sep_char for CSV
+     Input XLS:
+        --dtfmt=fmt Specify the default date format to replace 'm-d-yy'
+                    the default replacement is 'yyyy-mm-dd'
+     Output Text (default):
+        -s <sep>    Use separator <sep>. Default '|', \n allowed
+        -L          Line up the columns
+        -n          Number lines (prefix with column number)
+        -A          Show field attributes in ANSI escapes
+     Grep options:
+        -i          Ignore case
+        -w          Match whole words only
+        -h[#]       Show # header lines
+     Output CSV:
+        -c          Output CSV, separator = ','
+        -m          Output CSV, separator = ';'
+     Output HTML:
+        -H          Output HTML
+     Selection:
+        -S <sheets> Only print sheets <sheets>. 'all' is a valid set
+                    Default only prints the first sheet
+        -R <rows>   Only print rows    <rows>. Default is 'all'
+        -C <cols>   Only print columns <cols>. Default is 'all'
+        -F <flds>   Only fields <flds> e.g. -FA3,B16
+
+=head2 C<ss2tk>
+
+Show a spreadsheet in a perl/Tk spreadsheet widget
+
+ usage: ss2tk [-w <width>] [X11 options] file.xls [<pattern>]
+        -w <width> use <width> as default column width (4)
+
+=head2 C<xls2csv>
+
+Convert a spreadsheet to CSV. This is just a small wrapper over C<xlscat>.
+
+ usage: xls2csv [ -o file.csv ] file.xls
+
 =head1 TODO
 
 =over 4
@@ -995,45 +1172,49 @@ OO interface.
 
 =item Text::CSV_XS, Text::CSV_PP
 
-http://search.cpan.org/dist/Text-CSV_XS ,
-http://search.cpan.org/dist/Text-CSV_PP , and
-http://search.cpan.org/dist/Text-CSV .
+http://metacpan.org/release/Text-CSV_XS ,
+http://metacpan.org/release/Text-CSV_PP , and
+http://metacpan.org/release/Text-CSV .
 
 Text::CSV is a wrapper over Text::CSV_XS (the fast XS version) and/or
 Text::CSV_PP (the pure perl version)
 
 =item Spreadsheet::ParseExcel
 
-http://search.cpan.org/dist/Spreadsheet-ParseExcel
+http://metacpan.org/release/Spreadsheet-ParseExcel
+
+=item Spreadsheet::ParseXLSX
+
+http://metacpan.org/release/Spreadsheet-ParseXLSX
 
 =item Spreadsheet::XLSX
 
-http://search.cpan.org/dist/Spreadsheet-XLSX
+http://metacpan.org/release/Spreadsheet-XLSX
 
 =item Spreadsheet::ReadSXC
 
-http://search.cpan.org/dist/Spreadsheet-ReadSXC
+http://metacpan.org/release/Spreadsheet-ReadSXC
 
 =item Spreadsheet::BasicRead
 
-http://search.cpan.org/dist/Spreadsheet-BasicRead
+http://metacpan.org/release/Spreadsheet-BasicRead
 for xlscat likewise functionality (Excel only)
 
 =item Spreadsheet::ConvertAA
 
-http://search.cpan.org/dist/Spreadsheet-ConvertAA
+http://metacpan.org/release/Spreadsheet-ConvertAA
 for an alternative set of cell2cr () / cr2cell () pair
 
 =item Spreadsheet::Perl
 
-http://search.cpan.org/dist/Spreadsheet-Perl
+http://metacpan.org/release/Spreadsheet-Perl
 offers a Pure Perl implementation of a spreadsheet engine. Users that want
 this format to be supported in Spreadsheet::Read are hereby motivated to
 offer patches. It's not high on my TODO-list.
 
 =item xls2csv
 
-http://search.cpan.org/dist/xls2csv offers an alternative for my C<xlscat -c>,
+http://metacpan.org/release/xls2csv offers an alternative for my C<xlscat -c>,
 in the xls2csv tool, but this tool focuses on character encoding
 transparency, and requires some other modules.
 
@@ -1045,7 +1226,7 @@ H.Merijn Brand, <h.m.brand@xs4all.nl>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2005-2010 H.Merijn Brand
+Copyright (C) 2005-2014 H.Merijn Brand
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
